#. Distribuição Normal
#  Revisado: 2023-07-13 17:57:34 - J.C.Faria

#. Exemplo 1: Fenômeno que se ajusta a distribuição normal (apresentado nos slides)
#.. Conjunto de funções
#-----------------------------------------------------------------------------------
#... Função que lança as moedas e conta a quantidade de caras
count_heads <- function() {
  res <- sample(c('heads',
                  'tails'),
                size=200,
                replace=TRUE)
  
  sum(res == 'heads')
}

#install.packages('ggplot2') # Pacote para visualização do histograma.
library(ggplot2)

#... Histograma de densidade de results com cor de preenchimento dada por fill_color
histogram <- function(res, fill_color) {
  df <- data.frame(res)
  hist <- ggplot(df,
                 aes(x=res)) +
    geom_histogram(aes(y=after_stat(density)),
                   colour=1,
                   fill=fill_color,
                   binwidth=1,
    ) +
    xlab('Resultados = CARA') +
    ylab('Densidade')
  (hist)
}

#... Curva normal com média m e desvio padrão s
ncurve <- function(m, s) {
  (stat_function(fun=dnorm,
                 args=list(mean=m,
                           sd=s),
                 colour='#ff1765',
                 linewidth=2))
}

#... Função que gera o plot do histograma
make_plot <- function(res,
                      normal_curve=F) {
  # Histograma
  if(!normal_curve){ 
    hist <- histogram(res,
                      '#ff8eb4')
    hist
  }
  # Histograma + Curva Normal
  else {
    m <- 200*0.5
    s <- sqrt(200*0.5*0.5)
    
    hist <- histogram(res,
                      '#c5c5c5')
    curve <- ncurve(m,
                    s)
    
    hist + curve
  }
}
#-----------------------------------------------------------------------------------

#.. Usando as funções
res <- replicate(1e2,
                 count_heads()) # Teste para 100 lançamentos
make_plot(res)
make_plot(res,
          T) # Histograma + Curva Normal

res <- replicate(5e2,
                 count_heads()) # Teste para 500 lançamentos
make_plot(res)
make_plot(res,
          T) # Histograma + Curva Normal

res <- replicate(2e3,
                 count_heads()) # Teste para 2.000 lançamentos
make_plot(res)
make_plot(res,
          T) # Histograma + Curva Normal

# Com n=1e4 (10.000) associa rapidez computacional com boa convergência estatística
res <- replicate(1e4,
                 count_heads()) # Teste para 10.000 lançamentos
make_plot(res)
make_plot(res,
          T) # Histograma + Curva Normal

#... Bem demorado!!!
#res <- replicate(1e6,
#                 count_heads()) # Teste para 1.000.000 lançamentos
#make_plot(res)
#make_plot(res,
#          T) # Histograma + Curva Normal

#. Funções do R para operações com distribuições de probabilidades.
# Existem 4 operações básicas que combinam as letras (d, p, q, r) com o termo norm
# Por default as funções assumem a distribuição normal padrão 
# N(μ=0, σ=1).

# Considerando que X ~ N(μ, σ):
#... Densidade associada a um quantil X
str(dnorm) #d
dnorm(0) 

#... Probabilidade associada a um quantil Q
str(pnorm) #p
pnorm(0)   # P(X <= 0)
pnorm(0,
      lower.tail=FALSE) # P(X > 0)

#... Quantil associado a uma probabilidade P
str(qnorm) #q
qnorm(0.5)

#... Randômicos
str(rnorm) #r
rnorm(100)

#.. Normal feita na mão
fnorm <- function(x, m, s)
  (1 / (s * sqrt(2 * pi))) * exp(-1 / 2 * ((x - m) / s) ^ 2)

fnorm(x=0,
      m=0,
      s=1)

dnorm(x=0)

#.. Normal reduzida feita na mão
fz <- function(z)
  (1 / sqrt(2 * pi)) * exp((-z ^ 2 / 2))

fz(0)
dnorm(x=0)

#. Exemplo 2: Níveis de Colesterol
# Uma amostra n=250 do nível de colesterol para rapazes de 14 anos de idade,
# com média de 170mg por decilitro de sangue, e desvio padrão de 30 mg/dl.
amo <- rnorm(250,
             m=170,
             s=30) # Vetor aleatório contendo 250 amostras

# Histograma do vetor
hist(amo,
     freq=FALSE,
     col='#3998FF')

# Curva normal
curve(dnorm(x,
            m=170,
            s=30),
      col=2,
      lty=2,
      lwd=2,
      add=TRUE,
      n=1e3)

dnorm(150,
      m=170,
      s=30)

#---------------------------------------#
#. Formas da curva de densidade
#---------------------------------------#
#.. Fazendo variar a média
curve(dnorm(x,
            m=1,
            s=1.5),
      col='#3998FF',
      lwd=4,
      from=-6,
      to=17,
      n=1e3)

curve(dnorm(x,
            m=10,
            s=1.5),
      col='#FF659B',
      lwd=4,
      n=1e3,
      add=TRUE)

legend('topright',
       legend=c(expression(μ == 1),
                expression(μ == 10)),
       text.col=c('#3998FF',
                  '#FF659B'),
       cex=1.5,
       box.lty=0,
       inset=0.03,
       bg='transparent'
)
title('Distribuição normal com médias diferentes')

#.. Fazendo variar o desvio padrão
curve(dnorm(x,
            m=4,
            s=sqrt(3)),
      col='#3998FF',
      lwd=4,
      from=-15,
      to=25,
      n=1e3)

curve(dnorm(x,
            m=4,
            s=sqrt(20)),
      col='#FF659B',
      lwd=4,
      n=1e3,
      add=TRUE)

legend('topright',
       legend=c(expression(σ ^ 2 == 3),
                expression(σ ^ 2 == 20)),
       text.col=c('#3998FF',
                  '#FF659B'),
       cex=1.5,
       box.lty=0,
       inset=0.03,
       bg='transparent')
title('Distribuição normal com variâncias diferentes')

#.. Fazendo variar ambos
curve(dnorm(x,
            m=0,
            s=sqrt(0.2)),
      lwd=4,
      from=-5,
      to=5,
      n=1e3,
      ylab='f(x)',
      col='#FF659B')

curve(dnorm(x,
            m=0,
            s=sqrt(1.0)),
      col='#3998FF',
      lwd=4,
      n=1e3,
      add=TRUE)

curve(dnorm(x,
            m=0,
            s=sqrt(5)),
      col='#FF6500',
      lwd=4,
      n=1e3,
      add=TRUE)

curve(dnorm(x,
            m=-2,
            s=sqrt(0.5)),
      col='#00CD00',
      lwd=4,
      n=1e3,
      add=TRUE)

legend('topright',
       inset=0.05,
       col=c('#FF659B',
             '#3998FF',
             '#FF6500',
             '#00CD00'),
       legend=c('μ=0,  σ=0.2',
                'μ=0,  σ=1.0',
                'μ=0,  σ=5.0',
                'μ=-2, σ=0.5'),
       cex=1,
       lwd=4,
       box.lty=0,
       bg='transparent')

title('Distribuição normal com valores de μ e σ diferentes')

#. Exemplo 3
#... Função densidade de probabilidade: X ~ N(μ, σ)
fdp <- function(x,
                Mu=15,
                Sigma=2) {
  1 / (Sigma * sqrt(2 * pi)) * exp(-1 / 2 * ((x - Mu) / Sigma) ^ 2)
}

#... Visualizando a fdp
plot(fdp,
     xlim=c(5, 25),
     lwd=4,
     col='#FF659B'
)


legend('topright',
       legend=c(expression(μ == 15),
                expression(σ == 2)),
       text.col='#FF659B',
       cex=1.5,
       box.lty=0,
       inset=0.03)

#... Exemplo de probabilidade: P(X < 15)
integrate(fdp,
          low=-Inf,
          upp=15)$value

#... Exemplo de probabilidade: P(X < 13)
integrate(fdp,
          low=-Inf,
          upp=13)$value

#------------------------------------------------------#
#.. Pacote útil para destacar áreas do gráfico
#install.packages('PASWR')
library(PASWR)
#------------------------------------------------------#
# Utilizaremos a função 'normarea' para gerar os gráficos da normal
# Abaixo segue os argumentos da função 'normarea'

str(normarea)

#. Regra 'Empírica' (68-95-99.7)
#... Gráfico de P((μ - σ); μ + σ))
normarea(lower=-1, 
         upper=1, 
         m=0, 
         sig=1)
#... Gráfico de P((μ - 2σ); μ + 2σ))
normarea(lower=-2, 
         upper=2, 
         m=0, 
         sig=1)
#... Gráfico de P((μ - 3σ); μ + 3σ))
normarea(lower=-3, 
         upper=3, 
         m=0, 
         sig=1)
#... Gráfico de P((μ - 4σ); μ + 4σ))
normarea(lower=-4, 
         upper=4, 
         m=0, 
         sig=1)

#... Gráfico de P(X < 1.25)
normarea(lower=-Inf,
         upper=1.25,
         m=0,
         sig=1)

#------------------------------------------------------------#
#.. EXERCÍCIOS RELATIVOS A NORMAL PADRÃO: X ~ N(μ=0, σ=1)
#------------------------------------------------------------#
# a) P (X < 0.30)
normarea(low=-Inf,
         upp=0.30,
         m=0,
         sig=1)

pnorm(0.30)

# b) P (0 < X < 1.75)
normarea(low=0,
         upp=1.75,
         m=0,
         sig=1)

pnorm(1.75) - pnorm(0)

# c) P (1.25 < X < 1.82)
normarea(low=1.25,
         upp=1.82,
         m=0,
         sig=1)

pnorm(1.82) - pnorm(1.25)

# d) P (X > 1.8)
normarea(low=1.8,
         upp=Inf,
         m=0,
         sig=1)

1 - pnorm(1.8)

pnorm(1.8,
      lower.tail=FALSE)


#------------------------------------------------------#
#. Exemplo 4: APLICAÇÕES DA DISTRIBUIÇÃO NORMAL
#------------------------------------------------------#
# O tempo gasto no exame vestibular de uma universidade tem
# distribuição Normal, com μ=150 min e σ=30 min.
# Sorteando-se um aluno ao acaso, qual é a probabilidade
# dele terminar o exame antes de 120 minutos?

# P(X < 120)
normarea(lower=-Inf,
         upper=120,
         m=150,
         sig=30)


round(pnorm(120,
            m=150,
            s=30),
      4)


# Para o exemplo acima, suponha que queremos saber qual
# quantil fornece probabilidade 10%:
# dos menores valores
qnorm(10/100,
      m=150,
      s=30)

# dos maiores valores
qnorm(10/100,
      m=150,
      s=30,
      lower.tail=FALSE)
